import yaml
import os
import logging
from snowflake import connector


class SnowflakeObject:
    """Base class for all Snowflake Object types.
    This object holds the methods needed to connect to Snowflake and configure Snowflake credentials."""

    # True once Snowflake configured and sf_connection established
    sf_initialized = False
    # credentials used to establish Snowflake connection
    sf_credentials = None
    # Snowflake connection object
    sf_connection = False
    # Snowflake sf_cursor object
    sf_cursor = False
    # logger object
    _logger = logging.getLogger()
    # log message
    log_message = str
    # pandas dataframe of Snowflake table
    sf_df = None

    def initialize_snowflake(self, table: str, database: str = None, schema: str = None):
        """method that creates Snowflake connection and configures Snowflake credentials"""

        if not self.sf_initialized:
            # calls method to configure Snowflake credentials
            self.configure_sf_credentials(
                schema=schema,
                database=database
            )
            # calls method to connect to Snowflake using the sf_credentials variable
            self.get_snowflake_connection(
                user=self.sf_credentials['username'],
                pswd=self.sf_credentials['password'],
                acct=self.sf_credentials['account'],
                role=self.sf_credentials['role']
            )

            # calls method to read from snowflake table and return pandas dataframe
            self.get_sf_pandas_dataframe(
                table=table
            )

        self.sf_initialized = True

    def configure_sf_credentials(self, database: str = None, schema: str = None):
        """configures Snowflake credentials for session"""

        # Path to Snowflake credentials file
        __profile_path: str = os.path.join(os.getenv("HDV_HOME"),
                                           '.hashmap_data_validator/hdv_profiles.yml')
        with open(__profile_path, 'r') as stream:
            self.sf_credentials = yaml.safe_load(stream)['sink']['snowflake']

        # overwrite default credentials with passed in credentials if applicable
        if database:
            self.sf_credentials['database'] = database
        if schema:
            self.sf_credentials['schema'] = schema

        # role is None if none is given or configured as default
        if self.sf_credentials['role'] == '<your snowflake role>':
            self.sf_credentials['role'] = None

        # warehouse is None if none is given or configured as default
        if self.sf_credentials['warehouse'] == '<your snowflake warehouse>':
            self.sf_credentials['warehouse'] = None

        # checks if user has configured or passed credentials
        if self.sf_credentials['username'] == '<snowflake_username>' or self.sf_credentials['password'] == \
                '<snowflake_password>' or self.sf_credentials['account'] == '<snowflake_account>' \
                or self.sf_credentials['database'] == '<snowflake_database>' or self.sf_credentials['schema'] == \
                '<snowflake_schema>':

            self.log_message = f"Please configure your Snowlflake credentials at {__profile_path} or pass the applicable credentials " \
                               f"as arguments when calling this method."
            self._logger.error(self.log_message)
            self.sf_credentials = None
            return False

        return True

    def get_snowflake_connection(self, user, pswd, acct, role=None):
        """establishes a connection with snowflake"""
        self.sf_connection = connector.connect(
            user=user,
            password=pswd,
            account=acct,
            role=role

        )

        return True

    def get_sf_pandas_dataframe(self, table: str):
        """Reads data from a Snowflake table as a pandas dataframe."""

        # initialize cursor object
        self.sf_cursor = self.sf_connection.cursor()
        
        # use warehouse if not None
        if self.sf_credentials['warehouse']:
            self.sf_cursor.execute(f"use warehouse {self.sf_credentials['warehouse']};")

        # select the sf table
        self.sf_cursor.execute(f"select * from {self.sf_credentials['database']}.{self.sf_credentials['schema']}.{table};")

        # fetch sf table as pandas dataframe
        self.sf_df = self.sf_cursor.fetch_pandas_all()



import platform
import os
import yaml
from hdv.validator import Validator


if not os.getenv('HDV_HOME'):
    if platform.system().lower() != 'windows':
        os.environ['HDV_HOME'] = os.getenv('HOME')
    else:
        os.environ['HDV_HOME'] = os.getenv('USERPROFILE')

# Create configuration
profiles_path = os.path.join(os.getenv("HDV_HOME"), ".hashmap_data_validator/hdv_profiles.yml")
default_profiles_path: str = os.path.join(os.path.dirname(__file__),
                                          'configurations/default_hdv_profiles.yml')

#  If the configuration path does not exist - then a default configuration will be created
if not os.path.exists(profiles_path):

    # Set the path for the default configuration if it does not exist
    hdv_profiles = os.path.join(os.getenv("HDV_HOME"), ".hashmap_data_validator")
    if not os.path.exists(hdv_profiles):
        os.mkdir(hdv_profiles)

    # Load the default configuration
    with open(default_profiles_path, 'r') as default_stream:
        profiles_configuration = yaml.safe_load(default_stream)

    # Write the default configuration
    with open(profiles_path, 'w') as stream:
        _ = yaml.dump(profiles_configuration, stream)

# give users ability to call validate method from python file
hdv = Validator().hdv



import click
from hdv.validator import Validator


@click.command()
@click.option('-sf', '--snowflake_table', required=True, help='Snowflake table to be validated')
@click.option('-jdbc', '--jdbc_table', required=True, help='JDBC table to be validated')
def cli_validate(snowflake_table: str, jdbc_table: str):
    """ - makes hdv method callable via cli
        - configured in setup.py to be callable with 'hdv' command """

    # Validator instance
    cli_validation = Validator()

    # calls hdv method from cli
    cli_validation.hdv(snowflake_table=snowflake_table, jdbc_table=jdbc_table)



import great_expectations as ge
import pandas as pd
from json2html import Json2Html
import webbrowser
import os
from hdv.jdbc_objects.jdbc_object import JDBCObject
from hdv.snowflake_objects.snowflake_object import SnowflakeObject


class Validator(SnowflakeObject, JDBCObject):
    """Class that holds the validate method"""

    # row count validation
    row_count_valid = False
    # row hash validation
    row_hash_valid = False
    # great expectations snowflake dataframe
    ge_sf_df = None
    # hash df
    hash_df = None
    # json string to generate html report
    json_str = str

    def hdv(self, jdbc_table: str, snowflake_table: str, snowflake_database: str = None,
            snowflake_schema: str = None, jdbc_database: str = None):
        """validation method: runs expectations on row count and row hash values between Snowflake and JDBC tables"""

        try:
            # get jdbc pandas dataframe
            self.initialize_jdbc(database=jdbc_database,
                                 table=jdbc_table
                                 )

            # get snowflake pandas dataframe
            self.initialize_snowflake(database=snowflake_database,
                                      schema=snowflake_schema,
                                      table=snowflake_table
                                      )

            # run row count expectation on dataframes
            self.count_df_rows()

            # run row hash comparison expectation on dataframes
            self.compare_row_hashes()

            # create a json string from validation results
            self.json_str = '{"row_count_expectation": ' + self.row_count_valid + ', "row_hash_expectation": ' + self.row_hash_valid + "}"

            # generate html report from json string (.html file gets created from directory where the method is run)
            self.generate_html_report()

            # open the .html report
            webbrowser.open('file://' + os.path.realpath('expectation_report.html'))

        except Exception as e:
            self._logger.error(e)
            return False

        finally:
            # close connections
            if self.jdbc_connection:
                self.jdbc_connection.close()
            if self.sf_connection:
                self.sf_connection.close()
            if self.sf_cursor:
                self.sf_cursor.close()

    def generate_hash_list(self, df: pd.DataFrame):
        """generates a list of hash tuples over the rows in a dataframe"""
        hash_list = df.apply(lambda x: hash(tuple(x)), axis=1).tolist()

        return hash_list

    def count_df_rows(self):
        """expectation to count dataframe rows"""

        # create a great expectations dataframe from the snowflake dataframe for row count expectation
        self.ge_sf_df = ge.from_pandas(self.sf_df)

        # run row count expectation
        self.row_count_valid = str(self.ge_sf_df.expect_table_row_count_to_equal(len(self.jdbc_df.index)))

    def compare_row_hashes(self):
        """expectation to compare row hash strings"""

        # construct series from snowflake hash list
        sf_series = pd.Series(self.generate_hash_list(self.sf_df), name='sf_hashes')

        # construct series from jdbc hash list
        jdbc_series = pd.Series(self.generate_hash_list(self.jdbc_df), name='jdbc_hashes')

        # create the hash dataframe to be used in the hash expectation through concatenation
        self.hash_df = pd.concat([sf_series, jdbc_series], axis=1)

        # convert pandas dataframe to a great expectations dataframe
        self.hash_df = ge.from_pandas(self.hash_df)

        # run hash comparison expectation
        self.row_hash_valid = str(self.hash_df.expect_column_pair_values_to_be_equal(column_A='sf_hashes',
                                                                                     column_B='jdbc_hashes',
                                                                                     ignore_row_if="either_value_is_missing"))

    def generate_html_report(self):
        """converts json string to html and writes to html file"""

        # initiate json2html object
        j2h = Json2Html()

        # convert json to html
        html_str = j2h.convert(json=self.json_str)

        # write html string to file
        with open(os.path.realpath('expectation_report.html'), "w") as html_report:
            html_report.write(html_str)
            html_report.close()
        return



import yaml
import os
import logging
import jaydebeapi
import pandas as pd


class JDBCObject:
    """Base class for all JDBC Object types.
    This object holds the methods needed to connect to JDBC and configure JDBC credentials."""

    # True once JDBC configured and connection established
    jdbc_initialized = False
    # credentials used to establish JDBC connection
    jdbc_credentials = None
    # JDBC connection object
    jdbc_connection = False
    # logger object
    _logger = logging.getLogger()
    # log message
    log_message = str
    # jdbc pandas dataframe
    jdbc_df = None

    def initialize_jdbc(self, table: str, database: str = None):
        """method that creates JDBC connection and configures JDBC credentials"""

        if not self.jdbc_initialized:
            # calls method to configure JDBC credentials
            self.configure_jdbc_credentials(database=database)

            # calls method to connect to JDBC using the jdbc_credentials variable
            self.get_jdbc_connection()

            self.get_jdbc_pandas_dataframe(table=table)

        self.jdbc_initialized = True

    def configure_jdbc_credentials(self, database: str = None):
        """configures JDBC credentials for session"""

        # Path to JDBC credentials file
        __profile_path: str = os.path.join(os.getenv("HDV_HOME"),
                                           '.hashmap_data_validator/hdv_profiles.yml')
        with open(__profile_path, 'r') as stream:
            self.jdbc_credentials = yaml.safe_load(stream)['source']['jdbc']

        # overwrite default credentials with passed in credentials if applicable
        if database:
            self.jdbc_credentials['database'] = database

        # checks if user has configured or passed credentials
        if self.jdbc_credentials['user'] == '<username>' or self.jdbc_credentials['password'] == \
                '<password>' or self.jdbc_credentials['host'] == '<host>' or self.jdbc_credentials['port'] == '<port>'\
                or self.jdbc_credentials['driver'] == '<driver_name>' or self.jdbc_credentials['jar_file'] == \
                '<jar_file_path>' or self.jdbc_credentials['database'] == '<database_name>':

            self.log_message = f"Please configure JDBC your credentials at {__profile_path} or pass the applicable credentials " \
                               f"as arguments when calling this method."
            self._logger.error(self.log_message)
            self.jdbc_credentials = None
            return False

        return True

    def get_jdbc_connection(self):
        """establishes a connection with jdbc server"""
        url = self.jdbc_credentials['host'] + ':' + str(self.jdbc_credentials['port'])

        if self.jdbc_credentials['jar_file']:
            self.jdbc_connection = jaydebeapi.connect(self.jdbc_credentials['driver'],
                                                      url=url,
                                                      driver_args=[
                                                          self.jdbc_credentials['user'],
                                                          str(self.jdbc_credentials['password'])
                                                                   ],
                                                      jars=self.jdbc_credentials['jar_file']
                                                      )
        else:
            self.jdbc_connection = jaydebeapi.connect(self.jdbc_credentials['driver'],
                                                      url=url,
                                                      driver_args=[
                                                          self.jdbc_credentials['user'],
                                                          str(self.jdbc_credentials['password'])
                                                                   ]
                                                      )
        return True

    def get_jdbc_pandas_dataframe(self, table: str):
        """Reads data from a JDBC table as a pandas dataframe."""

        self.jdbc_df = pd.read_sql_query(sql=f"select * from {self.jdbc_credentials['database']}.{table}",
                                         con=self.jdbc_connection
                                         )



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="hashmap_data_validator",
    version="0.0.0.5",
    author="Hashmap, Inc",
    author_email="accelerators@hashmapinc.com",
    description="A Python Package designed to validate data sources and sinks",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/hashmapinc/ctso/accelerators/data-engineering/hashmap_data_suite/hashmap-data-validator",
    packages=setuptools.find_packages(),
    package_data={
        "hdv": ["configurations/default_hdv_profiles.yml"],
    },
    install_requires=[
        'pyarrow==0.17.1',
        'pandas==1.1.4',
        'pyyaml==5.3.1',
        'snowflake-connector-python==2.3.6',
        'great-expectations==0.13.6',
        'JayDeBeApi==1.2.3'
    ],
    entry_points={
        'console_scripts': [
            'hdv = hdv.cli_validator:cli_validate'
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Development Status :: 3 - Alpha"
    ],
    python_requires='>=3.7',
)